game_engine:
	;wait for frame start here
	stz frame_ready
	-
		wai
		lda frame_ready
	beq -
start_game_engine:
	%jsr_indirect(game_prepare)
	%jsr_indirect(game_mode)
	%jsr_indirect(nmi_prepare)
	
	bra game_engine
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Game mode management routines
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


init_mode_7:
	lda #$8080
	sta oam_priority_settings

	%dma_fblank_mode_7(mode_7_tilemaps, mode_7_graphics)
	%dma_fblank_palette(mode_7_palettes, $00, $80)
	
	jsr mode_7_setup
	jsr enable_screen

	%set_game_mode(run_mode_7)
	%set_nmi_mode(nmi_mode_7)
	rts
	
run_mode_7:
	jsr update_controllers
	jsr update_mode_7_settings
	rts


prepare_general_nmi_dma:
	rep #$30
	jsr prepare_oam
game_mode_return:
	rts
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Below are game mode implementation routines
; Above should be mostly simple and glue logic of the below
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	
mode_7_setup:
	sep #$20
	
	;enable mode 7
	lda #$07
	sta $2105
	
	lda #$00 
	sta $2107
	
	stz $210B
	
	lda #$00
	sta $211a
	
	rep #$20
	
	lda #$0100
	sta mode_7_A
	sta mode_7_D
	sta mode_7_scale_x
	sta mode_7_scale_y
	
	sep #$20
	
	;set up mode 7 default parameters
	sta $211B
	xba	
	sta $211B
	xba
	stz $211C
	stz $211C
	stz $211D
	stz $211D
	sta $211E
	xba      
	sta $211E
	
	;set center to 0,0
	stz $211F
	stz $211F
	stz $2120
	stz $2120
	
	stz $210D
	stz $210D
	
	stz $210E
	stz $210E

	lda #$01            ; Enable BG1
	sta $212C
	
	rep #$20
	
	stz layer_1_x
	stz layer_1_y
	rts
	
update_mode_7_settings:
	lda controller_active
	bit !button_up
	beq +
		dec layer_1_y
		dec layer_1_y
	+
	
	bit !button_down
	beq +
		inc layer_1_y
		inc layer_1_y
	+
	
	bit !button_left
	beq +
		dec layer_1_x
		dec layer_1_x
	+
	
	bit !button_right
	beq +
		inc layer_1_x
		inc layer_1_x
	+
	
	bit !button_Y
	beq +
		inc mode_7_scale_x
		inc mode_7_scale_x
	+
	
	bit !button_X
	beq +
		dec mode_7_scale_x
		dec mode_7_scale_x
	+
	
	bit !button_B
	beq +
		inc mode_7_scale_y
		inc mode_7_scale_y
	+
	
	bit !button_A
	beq +
		dec mode_7_scale_y
		dec mode_7_scale_y
	+
	
	bit !button_L
	beq +
		inc mode_7_rotate
	+
	
	bit !button_R
	beq +
		dec mode_7_rotate
	+
	rts
	
enable_screen:
	lda #$000F
	sta brightness

	lda #$0081
	sta $4200
	rts
	
	
prepare_oam:
	stz $4330	
	stz $2132	
	lda #$0004	
	sta $4331	
	lsr
	sta $4333	
	lda #$0220	
	sta $4335
	
	lda !channel_3
	tsb dma_hdma_mirror
	rts
