reset:
	sei 	  
	clc 	  
	xce 	  
	stz $4200
 	
 	;setup stack/direct page/clear decimal
 	rep #$38 
 	lda #stack
 	tcs 	 
 	lda #$0000 
 	tcd

 	jsr register_init
	jsr clear_RAM
	
	ldx #$80
 	stx brightness

	%set_game_prepare(game_mode_return)
	%set_game_mode(init_mode_7)
	%set_nmi_prepare(prepare_general_nmi_dma)
	%set_nmi_mode(nmi_return)
	%set_irq_mode(irq_return)
	
	cli
	rep #$30
 	
	jmp start_game_engine
	
register_init:  
 	sep #$10
 	rep #$20
 	ldx #$80 	;\ enable force blank
 	stx $2100	;/
 	stz $2101	; use 16x16 and 8x8 sprites, graphics at vram $0000
 	stz $2102	; reset oam address
 	
 	ldx #$01	
 	stx $2105	; Mode 1
 	stz $2106	; disable mosaic
 	stz $2107	; clear 1/2 map VRAM location
 	stz $2109	; clear 3/4 map VRAM location
 	stz $210B	; clear 1/2/3/4 Tile data location
 	
 	dex
 	stx $210D	;\ layer 1 X
 	stx $210D	;/
 	stx $210F	;\ layer 2 X
 	stx $210F	;/
 	stx $2111	;\ layer 3 X
 	stx $2111	;/
 	stx $2113	;\ layer 4 X
 	stx $2113	;/
 	
 	ldy #$07	;\ 1 pixel down so you can see the top
 	dex		;/
 	stx $210E	;\ layer 1 Y
 	sty $210E	;/
 	stx $2110	;\ layer 2 Y
 	sty $2110	;/
 	stx $2112	;\ layer 3 Y
 	sty $2112	;/
 	stx $2114	;\ layer 4 Y
 	sty $2114	;/

 	stz $2123	; clear window masks
 	stz $2125	; clear color masks
 	stz $2126	; Clear window 1/2 positions
 	stz $2128	; Clear window 3/4 positions
 	stz $212A	; Clear windowing logic
 	
 	lda #$0013 	; enable sprites, layer 1, layer 2, clear subscreen
 	sta $212C	; Main Screen designation (planes, sprites enable)
 	
 	stz $212E	; Window mask for Main Screen
 	lda #$0030
 	sta $2130	; Disable color math
 	
 	lda #$00E0	
 	sta $2132	; reset color intensity and clear screen mode
 	rts

clear_RAM:
	jsr clear_wram
	jsr clear_vram
	jsr clear_cgram
	jsr clear_oam
	rts

clear_wram:
	stz $2181
	stz $2183
	
	lda #$8008
	sta $4300
	lda #.fill_byte
	sta $4302
	ldx #.fill_byte>>16
	stx $4304
	lda #stack-4	;Clear up to the last 4 bytes of stack
	sta $4305
	ldx #$01
	stx $420B

	lda #$2000
	sta $2181
	stz $2183
		
	lda #$E000
	sta $4305
	ldx #$01
	stx $420B
	
	stz $4305
	ldx #$01
	stx $420B
	
	rts
	
.fill_byte
	db $00


clear_vram:
	ldx #$80
	stx $2115    
	lda #$1809
	sta $4300    
	stz $2116    
	stz $4302
	stz $4304    
	lda #$FFFF
	sta $4305    
	
	ldx #$01
	stx $420B    
	
	stz $2119 
	rts

clear_cgram:
	stz $2121
	ldy #$00
	ldx #$00
	-
		sty $2122
		sty $2122
		dex
	bne -
	rts
	
clear_oam:
	stz $2102
	ldx #$80
	ldy #$F0
	-
		stz $2104
		sty $2104
		stz $2104
		stz $2104
		dex
	bne -
	
	ldx #$20
	-
		stz $2104
		dex
	bne -
	rts
